﻿define([
// libs
    'jquery',
// live monitor
    'utility',

    'components/ComponentFactory',
// resources
    'text!components/Statistic/Templates/StatisticItem.html'
],

function (
// libs
    $,
// live monitor
    utility,

    componentFactory,
// resources
    templateString
) {

    // =================================================================================================================================================
    // 'LiveMonitorStatistic' class information
    // =================================================================================================================================================
    // module:
    //      'App/Components/Statistic/LiveMonitorStatistic'
    // summary:
    //      The jQuery plugin component for the statistic item
    // description:
    //      use:
    //          $(target).LiveMonitorStatistic(options);
    //      options:
    //          resources [Object]
    //          templateString [String]
    //          baseClasses [String]
    // tags:
    //      public

    // Create the defaults once
    var pluginName = 'LiveMonitorStatistic',
        pluginOptions = {
            templateString: templateString,
            baseClasses: 'livemonitor-statistic'
        },
        pluginDefinitions = {

            // =================================================================================================================================================
            // Public functions
            // =================================================================================================================================================

            bindData: function (/*Object*/statisticsData) {
                // summary:
                //      Bind statistics data for the current component
                // statisticsData: [Object]
                //      The given statistics data
                // tags:
                //      public, extensions

                this._setStatisticNumber(this._getStatisticNumber());
            },

            // =================================================================================================================================================
            // Overrided functions
            // =================================================================================================================================================

            _postInit: function () {
                // summary:
                //      Post-initialize settings for the component
                // tags:
                //      protected, extension

                this._setupLayout();
            },

            // =================================================================================================================================================
            // Protected functions
            // =================================================================================================================================================

            _getStatisticNumber: function () {
                // summary:
                //      Get the calculated statistic number
                // returns: [Integer]
                //      The statistic number
                // tags:
                //      protected, extensions

                return 0;
            },

            // =================================================================================================================================================
            // Private functions
            // =================================================================================================================================================

            _setupLayout: function () {
                // summary:
                //      Initialize default layout settings for the component
                // tags:
                //      private

                this._$number = this._$wrapper.find('.statistic-number');

                var $name = this._$wrapper.find('.statistic-name'),
                    statisticName = this._resources.statisticname;

                this._$number && this._$number.text(0);
                (statisticName && $name) && $name.text(statisticName);
            },

            _setStatisticNumber: function (/*Integer*/number) {
                // summary:
                //      Update statistic number when onReceiveCompleted fired
                // number: [Integer]
                //      The given integer number that wanted to set as the statistic number
                // tags:
                //      private

                this._$number && this._$number.text(isNaN(number) ? 0 : number);
            }
        };

    // A really lightweight plugin wrapper around the constructor, 
    //  preventing against multiple instantiations
    $.fn[pluginName] = function (/*Object*/options) {
        // summary:
        //      Create a new jQuery plugin from the given settings
        // options: [Object]
        //      The given settings that wanted to decorates the default settings of the current component
        // returns: [Object]
        //      The jQuery plugin object
        // tags:
        //      protected

        // Decorates the default settings by the given settings
        $.extend(true, pluginOptions, options);

        // Create and then return a new jQuery plugin object
        return componentFactory.create(this, pluginName, pluginDefinitions, pluginOptions);
    };

    $.fn[pluginName].prototype.pluginOptions = pluginOptions;
    $.fn[pluginName].prototype.pluginDefinitions = pluginDefinitions;

});