﻿define([
// libs
    'jquery',
// live monitor
    'utility'
],

function (
// libs
    $,
// live monitor
    utility
) {

    // =================================================================================================================================================
    // 'GroupElementBuilder' class information
    // =================================================================================================================================================
    // module:
    //      'App/Components/Svg/Factory/Builder/GroupElementBuilder'
    // summary:
    //      The element builder class for 'g' SVG object
    // description:
    //      Create the 'GROUP' SVG element
    // tags:
    //      public

    var GroupElementBuilder = {

        elementName: 'g',

        create: function (/*Object*/groupSettings, /*Array*/container, /*Function?*/recursiveCreate) {
            // summary:
            //      Create and then append a SVG group element ('g') to the given container
            // groupSettings: [Object]
            //      The given GROUP settings
            // container: [Array]
            //      The given container
            // recursiveCreate: [Function?]
            //      Indicated that should or should not create children recursively
            // tags:
            //      public

            if (!groupSettings) {
                return;
            }

            var groupList = container.selectAll(this.elementName)
                        .data(utility.getItemCollection(groupSettings))
                            .enter()
                    .append(this.elementName)
                        .attr('class', function (/*Object*/d) {
                            // d: [Object]
                            //      The given datum object

                            return d.classes;
                        })
                        .attr('height', function (/*Object*/d) {
                            // d: [Object]
                            //      The given datum object

                            return d.height;
                        })
                        .attr('width', function (/*Object*/d) {
                            // d: [Object]
                            //      The given datum object

                            return d.width;
                        });

            $.isFunction(recursiveCreate) && recursiveCreate(groupSettings, /*container*/groupList);
        }

    };

    return GroupElementBuilder;

});