﻿define([
// libs
    'jquery',
// live monitor
    'utility'
],

function (
// libs
    $,
// live monitor
    utility
) {

    // =================================================================================================================================================
    // 'PathElementBuilder' class information
    // =================================================================================================================================================
    // module:
    //      'App/Components/Svg/Factory/Builder/PathElementBuilder'
    // summary:
    //      The element builder class for 'path' SVG object
    // description:
    //      Create the 'PATH' SVG element
    // tags:
    //      public

    var PathElementBuilder = {

        elementName: 'path',

        create: function (/*Object*/pathSettings, /*Array*/container, /*Function?*/recursiveCreate) {
            // summary:
            //      Create and then append a SVG path element ('path') to the given container
            // pathSettings: [Object]
            //      The given PATH settings
            // container: [Array]
            //      The given container
            // recursiveCreate: [Function?]
            //      Indicated that should or should not create children recursively
            // tags:
            //      public

            if (!pathSettings) {
                return;
            }

            var pathList = container.selectAll(this.elementName)
                        .data(utility.getItemCollection(pathSettings))
                            .enter()
                    .append(this.elementName)
                        .attr('d', function (/*Object*/d) {
                            // d: [Object]
                            //      The given datum object

                            return d.d;
                        });

            $.isFunction(recursiveCreate) && recursiveCreate(pathSettings, /*container*/pathList);
        }

    };

    return PathElementBuilder;

});