﻿define([
// libs
    'jquery',
// live monitor
    'components/Svg/Factory/Template/BaseTemplate'
],

function (
// libs
    $,
// live monitor
    baseTemplate
) {

    // =================================================================================================================================================
    // 'HierarchicalBallTemplate' class information
    // =================================================================================================================================================
    // module:
    //      'App/Components/Svg/Factory/Template/HierarchicalBallTemplate'
    // summary:
    //      Provides hierarchical ball content markup template in SVG format
    // description:
    //      Public functions:
    //          load(/*String?*/text)
    //              Load SVG in JSON format
    //          loadExternal()
    //              Load SVG in raw format (SVG markup string)
    // tags:
    //      public

    var HierarchicalBallTemplate = {

        // key: [String] public
        //      The unique key that used to register to the controller
        //      The controller used this key to identifies each registered template
        key: 'HierarchicalBall',

        // name: [String] public
        //      The template name that used to register to the controller
        //      The application used this name to display
        name: 'Hierarchical Ball Template',

        // =================================================================================================================================================
        // Public functions
        // =================================================================================================================================================

        load: function (/*String?*/text) {
            // summary:
            //      Get predefined markup settings for a tree node SVG object
            // text: [String?]
            //      The given text that wanted to bind to the 'text' SVG tag
            // returns: [Object]
            //      The JSON markup settings for a tree node SVG object
            // tags:
            //      public

            return {
                svg: {
                    height: 40,
                    width: 40,
                    g: {
                        classes: 'livemonitor-ball hierarchical-content',
                        g: [
                            {
                                circle: {
                                    cx: 20,
                                    cy: 20,
                                    r: 20,
                                    fill: 'url(#livemonitor-verticalLinearGradient)'
                                },
                                ellipse: {
                                    cx: 20,
                                    cy: 13,
                                    rx: 15,
                                    ry: 10
                                }
                            },
                            {
                                circle: {
                                    classes: 'center-point',
                                    cx: 20,
                                    cy: 20,
                                    r: 3
                                },
                                text: {
                                    x: 20,
                                    y: 49,
                                    classes: 'name',
                                    value: text
                                }
                            },
                            {
                                circle: {
                                    classes: 'ring',
                                    cx: 20,
                                    cy: 20,
                                    r: 18
                                }
                            },
                            {
                                classes: 'collapsible-container',
                                height: 40,
                                width: 40,
                                text: {
                                    classes: 'collapsible-controller',
                                    value: '+',
                                    x: 22,
                                    y: 15
                                }
                            }
                        ]
                    }
                }
            };
        }

        // =================================================================================================================================================
        // Private functions
        // =================================================================================================================================================

    };

    return $.extend(true, {}, baseTemplate, HierarchicalBallTemplate);

});