﻿define([
// libs
    'jquery',
// live monitor
    'utility',

    'components/ComponentFactory',

    'components/Svg/ExternalTemplatePreloader',
    'components/Svg/Layout/MultiFociLayout'
],

function (
// libs
    $,
// live monitor
    utility,

    componentFactory,

    externalTemplatePreloader,
    multiFociLayout
) {

    // =================================================================================================================================================
    // 'LiveMonitorMultiFociLayoutWrapper' class information
    // =================================================================================================================================================
    // module:
    //      'App/Components/Tree/LiveMonitorMultiFociLayoutWrapper'
    // summary:
    //      The jQuery plugin for the multi-foci (SVG) layout wrapper component
    // description:
    //      use:
    //          $(target).LiveMonitorMultiFociLayoutWrapper(options);
    //      options:
    //          resources [Object]
    //          baseClasses [String]
    // tags:
    //      public

    // Create the defaults once
    var pluginName = 'LiveMonitorMultiFociLayoutWrapper',
        pluginOptions = {
            baseClasses: 'livemonitor-layer livemonitor-multiFociLayoutWrapper'
        },
        pluginDefinitions = {

            // =================================================================================================================================================
            // Private variables
            // =================================================================================================================================================

            // _svgTemplateKey: [String] private
            //      The key that used to get information about SVG template
            _svgTemplateKey: 'VisitorBall',

            // =================================================================================================================================================
            // Overrided functions
            // =================================================================================================================================================

            _postInit: function () {
                // summary:
                //      Post-initialize settings for the component
                // tags:
                //      protected, extension

                this._setupLayout();
                this._setupEvents();
            },

            // =================================================================================================================================================
            // Public callback functions
            // =================================================================================================================================================

            bindData: function (/*Object*/hierarchicalMappingData, /*Array*/visitorDataList) {
                // summary:
                //      Binding the given data to the existing hierarchical layout component
                // hierarchicalMappingData: [Object]
                //      The given mapping data object
                // visitorDataList: [Array]
                //      The given visitors data that will be used to bind to the multi foci layout control
                // tags:
                //      public, extensions

                $.when(externalTemplatePreloader.getTemplate(this._svgTemplateKey))
                    .done(utility.hitch(this, function (/*String*/templateString) {
                        this._multiFociLayout.bindData({
                            hierarchicalMappingData: hierarchicalMappingData,
                            visitorDataList: visitorDataList,
                            templateString: {
                                visitorBalls: this._getTemplateList(visitorDataList, templateString)
                            }
                        });
                    }));
            },

            // =================================================================================================================================================
            // Private events
            // =================================================================================================================================================

            _onOnlineVisitorSelected: function (/*Event*/evt, /*Object*/selectedVisitor) {
                // summary:
                //      Fired when a online visitor is selected on the online visitors list
                // evt: [Event]
                //      The given event
                // selectedVisitor: [Object]
                //      The given selected visitor object
                // tags:
                //      private

                if (selectedVisitor && selectedVisitor.visitorNumber) {
                    this._multiFociLayout && this._multiFociLayout.setSelection(selectedVisitor.visitorNumber);
                }
            },

            // =================================================================================================================================================
            // Private functions
            // =================================================================================================================================================

            _setupLayout: function () {
                // summary:
                //      Setup layout for the current container
                // tags:
                //      private

                this._multiFociLayout = multiFociLayout.create({
                    placeHolder: this.element,
                    animationKey: this.getSetting('animation')
                });
            },

            _setupEvents: function () {
                // summary:
                //      Setup events for the current component
                // tags:
                //      private

                this.bindContextEvent('onlineVisitorSelected', utility.hitch(this, this._onOnlineVisitorSelected));
            },

            _getTemplateList: function (/*Array*/collection, /*String*/templateString) {
                // summary:
                //      Build template collection from the given template string
                // collection: [Array]
                //      The given collection that used to generate template list
                // templateString: [String]
                //      The given template string that used to build the template collection
                // tags:
                //      private

                var totalItems = collection.length,
                    item,
                    templateList = [];

                while (totalItems--) {
                    item = collection[totalItems];
                    if (!item) {
                        continue;
                    }

                    this._injectChannelInfo(item);
                    templateList.unshift($(utility.formatTemplateString(templateString, item))[0]);
                }

                return templateList;
            }
        };

    // A really lightweight plugin $wrapper around the constructor, 
    //  preventing against multiple instantiations
    $.fn[pluginName] = function (/*Object*/options) {
        // summary:
        //      Create a new jQuery plugin from the given settings
        // options: [Object]
        //      The given settings that wanted to decorates the default settings of the current component
        // returns: [Object]
        //      The jQuery plugin object
        // tags:
        //      protected

        // Decorates the default settings by the given settings
        $.extend(true, pluginOptions, options);

        // Create and then return a new jQuery plugin object
        return componentFactory.create(this, pluginName, pluginDefinitions, pluginOptions);
    };

});